@@ -10,21 +10,19 @@
 #
 import unittest
 import logging
-import tempfile
 import warnings
 import pathlib
 import pickle
 import platform
-import glob
 import os
-import re
 from textwrap import dedent
 from xml.etree.ElementTree import Element
 
 import xmlschema
 from xmlschema import XMLSchemaParseError, XMLSchemaIncludeWarning, XMLSchemaImportWarning
 from xmlschema.names import XML_NAMESPACE, LOCATION_HINTS, SCHEMAS_DIR, XSD_ELEMENT, XSI_TYPE
-from xmlschema.validators import XMLSchemaBase, XMLSchema10, XMLSchema11, XsdGlobals
+from xmlschema.validators import XMLSchemaBase, XMLSchema10, XMLSchema11, \
+    XsdGlobals, XsdComponent
 from xmlschema.testing import SKIP_REMOTE_TESTS, XsdValidatorTestCase
 from xmlschema.validators.schemas import logger
 
@@ -34,7 +32,7 @@ class CustomXMLSchema(XMLSchema10):
 
 
 class TestXMLSchema10(XsdValidatorTestCase):
-    TEST_CASES_DIR = os.path.join(os.path.dirname(__file__), '../test_cases')
+    TEST_CASES_DIR = str(pathlib.Path(__file__).parent.joinpath('../test_cases').resolve())
     maxDiff = None
 
     class CustomXMLSchema(XMLSchema10):
@@ -217,6 +215,15 @@ class TestXMLSchema10(XsdValidatorTestCase):
         self.assertIsNotNone(xsd_type._annotation)  # xs:simpleType annotations are not lazy parsed
         self.assertEqual(str(xsd_type.annotation), ' stuff ')
 
+    def test_components(self):
+        components = self.col_schema.components
+        self.assertIsInstance(components, dict)
+        self.assertEqual(len(components), 25)
+
+        for elem, component in components.items():
+            self.assertIsInstance(component, XsdComponent)
+            self.assertIs(elem, component.elem)
+
     def test_annotation_string(self):
         schema = self.check_schema("""
             <xs:element name='A'>
@@ -640,9 +647,11 @@ class TestXMLSchema10(XsdValidatorTestCase):
         self.assertEqual(len(schema.maps.namespaces['']), 1)
 
         # Less checks on duplicate objects for schemas added after the build
-        schema.add_schema(source2, build=True)
-        self.assertEqual(len(schema.maps.namespaces['']), 2)
-        self.assertTrue(schema.maps.built)
+        with self.assertRaises(XMLSchemaParseError) as ctx:
+            schema.add_schema(source2, build=True)
+
+        self.assertIn("global element with name='elem2' is already defined",
+                      str(ctx.exception))
 
         with self.assertRaises(XMLSchemaParseError) as ec:
             schema.maps.clear()
@@ -653,13 +662,13 @@ class TestXMLSchema10(XsdValidatorTestCase):
         schema.add_schema(source2, namespace='http://xmlschema.test/ns', build=True)
         self.assertEqual(len(schema.maps.namespaces['http://xmlschema.test/ns']), 2)
 
-        # Need a rebuild to add elem2 from added schema ...
-        self.assertEqual(len(schema.elements), 1)
+        # Don't need a full rebuild to add elem2 from added schema ...
+        self.assertEqual(len(schema.elements), 2)
         schema.maps.clear()
         schema.build()
         self.assertEqual(len(schema.elements), 2)
 
-        # ... so is better to build after sources additions
+        # Or build after sources additions
         schema = self.schema_class(source1, build=False)
         schema.add_schema(source2, namespace='http://xmlschema.test/ns')
         schema.build()
@@ -670,142 +679,6 @@ class TestXMLSchema10(XsdValidatorTestCase):
         self.assertEqual(len(schema.maps.namespaces['http://xmlschema.test/ns1']), 1)
         self.assertEqual(len(schema3.elements), 1)
 
-    def test_export_errors__issue_187(self):
-        with self.assertRaises(ValueError) as ctx:
-            self.vh_schema.export(target=self.vh_dir)
-
-        self.assertIn("target directory", str(ctx.exception))
-        self.assertIn("is not empty", str(ctx.exception))
-
-        with self.assertRaises(ValueError) as ctx:
-            self.vh_schema.export(target=self.vh_xsd_file)
-
-        self.assertIn("target", str(ctx.exception))
-        self.assertIn("is not a directory", str(ctx.exception))
-
-        with self.assertRaises(ValueError) as ctx:
-            self.vh_schema.export(target=self.vh_xsd_file + '/target')
-
-        self.assertIn("target parent", str(ctx.exception))
-        self.assertIn("is not a directory", str(ctx.exception))
-
-        with tempfile.TemporaryDirectory() as dirname:
-            with self.assertRaises(ValueError) as ctx:
-                self.vh_schema.export(target=dirname + 'subdir/target')
-
-            self.assertIn("target parent directory", str(ctx.exception))
-            self.assertIn("does not exist", str(ctx.exception))
-
-    def test_export_same_directory__issue_187(self):
-        with tempfile.TemporaryDirectory() as dirname:
-            self.vh_schema.export(target=dirname)
-
-            for filename in os.listdir(dirname):
-                with pathlib.Path(dirname).joinpath(filename).open() as fp:
-                    exported_schema = fp.read()
-                with pathlib.Path(self.vh_dir).joinpath(filename).open() as fp:
-                    original_schema = fp.read()
-
-                if platform.system() == 'Windows':
-                    exported_schema = re.sub(r'\s+', '', exported_schema)
-                    original_schema = re.sub(r'\s+', '', original_schema)
-
-                self.assertEqual(exported_schema, original_schema)
-
-        self.assertFalse(os.path.isdir(dirname))
-
-    def test_export_another_directory__issue_187(self):
-        vh_schema_file = self.casepath('issues/issue_187/issue_187_1.xsd')
-        vh_schema = self.schema_class(vh_schema_file)
-
-        with tempfile.TemporaryDirectory() as dirname:
-            vh_schema.export(target=dirname)
-
-            path = pathlib.Path(dirname).joinpath('examples/vehicles/*.xsd')
-            for filename in glob.iglob(pathname=str(path)):
-                with pathlib.Path(dirname).joinpath(filename).open() as fp:
-                    exported_schema = fp.read()
-
-                basename = os.path.basename(filename)
-                with pathlib.Path(self.vh_dir).joinpath(basename).open() as fp:
-                    original_schema = fp.read()
-
-                if platform.system() == 'Windows':
-                    exported_schema = re.sub(r'\s+', '', exported_schema)
-                    original_schema = re.sub(r'\s+', '', original_schema)
-
-                self.assertEqual(exported_schema, original_schema)
-
-            with pathlib.Path(dirname).joinpath('issue_187_1.xsd').open() as fp:
-                exported_schema = fp.read()
-            with open(vh_schema_file) as fp:
-                original_schema = fp.read()
-
-            if platform.system() == 'Windows':
-                exported_schema = re.sub(r'\s+', '', exported_schema)
-                original_schema = re.sub(r'\s+', '', original_schema)
-
-            self.assertNotEqual(exported_schema, original_schema)
-            self.assertEqual(
-                exported_schema,
-                original_schema.replace('../..', dirname.replace('\\', '/'))
-            )
-
-        self.assertFalse(os.path.isdir(dirname))
-
-    @unittest.skipIf(SKIP_REMOTE_TESTS, "Remote networks are not accessible.")
-    def test_export_remote__issue_187(self):
-        vh_schema_file = self.casepath('issues/issue_187/issue_187_2.xsd')
-        vh_schema = self.schema_class(vh_schema_file)
-
-        with tempfile.TemporaryDirectory() as dirname:
-            vh_schema.export(target=dirname)
-
-            with pathlib.Path(dirname).joinpath('issue_187_2.xsd').open() as fp:
-                exported_schema = fp.read()
-            with open(vh_schema_file) as fp:
-                original_schema = fp.read()
-
-            if platform.system() == 'Windows':
-                exported_schema = re.sub(r'\s+', '', exported_schema)
-                original_schema = re.sub(r'\s+', '', original_schema)
-
-            self.assertEqual(exported_schema, original_schema)
-
-        self.assertFalse(os.path.isdir(dirname))
-
-        with tempfile.TemporaryDirectory() as dirname:
-            vh_schema.export(target=dirname, save_remote=True)
-            path = pathlib.Path(dirname).joinpath('brunato/xmlschema/master/tests/test_cases/'
-                                                  'examples/vehicles/*.xsd')
-
-            for filename in glob.iglob(pathname=str(path)):
-                with pathlib.Path(dirname).joinpath(filename).open() as fp:
-                    exported_schema = fp.read()
-
-                basename = os.path.basename(filename)
-                with pathlib.Path(self.vh_dir).joinpath(basename).open() as fp:
-                    original_schema = fp.read()
-                self.assertEqual(exported_schema, original_schema)
-
-            with pathlib.Path(dirname).joinpath('issue_187_2.xsd').open() as fp:
-                exported_schema = fp.read()
-            with open(vh_schema_file) as fp:
-                original_schema = fp.read()
-
-            if platform.system() == 'Windows':
-                exported_schema = re.sub(r'\s+', '', exported_schema)
-                original_schema = re.sub(r'\s+', '', original_schema)
-
-            self.assertNotEqual(exported_schema, original_schema)
-            self.assertEqual(
-                exported_schema,
-                original_schema.replace('https://raw.githubusercontent.com',
-                                        dirname.replace('\\', '/') + '/raw.githubusercontent.com')
-            )
-
-        self.assertFalse(os.path.isdir(dirname))
-
     def test_pickling_subclassed_schema__issue_263(self):
         cases_dir = pathlib.Path(__file__).parent.parent
         schema_file = cases_dir.joinpath('test_cases/examples/vehicles/vehicles.xsd')
@@ -827,23 +700,19 @@ class TestXMLSchema10(XsdValidatorTestCase):
         schema = CustomLocalXMLSchema(str(schema_file))
         self.assertTrue(schema.is_valid(str(xml_file)))
 
-        with self.assertRaises((pickle.PicklingError, AttributeError)) as ec:
+        with self.assertRaises((pickle.PicklingError, AttributeError)) as ec:  # type: ignore
             pickle.dumps(schema)
-        self.assertIn("Can't pickle", str(ec.exception))
 
-    def test_deprecated_check_schema_method(self):
+        error_message = str(ec.exception)
+        self.assertTrue(
+            "Can't get local object" in error_message or "Can't pickle" in error_message
+        )
 
-        with warnings.catch_warnings(record=True) as ctx:
-            warnings.simplefilter("always")
+    def test_meta_schema_validation(self):
+        self.assertTrue(self.schema_class.meta_schema.is_valid(self.vh_xsd_file))
 
-            self.schema_class.check_schema(self.vh_schema)
-
-            self.assertEqual(len(ctx), 1, "Expected one import warning")
-            self.assertIn("check_schema() class method will be removed in v3.0",
-                          str(ctx[0].message))
-
-        with self.assertRaises(RuntimeError):
-            self.schema_class.meta_schema.check_schema(self.vh_schema)
+        invalid_xsd = self.casepath('examples/vehicles/invalid.xsd')
+        self.assertFalse(self.schema_class.meta_schema.is_valid(invalid_xsd))
 
     def test_default_namespace_mapping__issue_266(self):
         schema_file = self.casepath('issues/issue_266/issue_266b-1.xsd')
@@ -854,6 +723,61 @@ class TestXMLSchema10(XsdValidatorTestCase):
         self.assertIn("the QName 'testAttribute3' is mapped to no namespace", error_message)
         self.assertIn("requires that there is an xs:import statement", error_message)
 
+    @unittest.skipIf(SKIP_REMOTE_TESTS, "Remote networks are not accessible.")
+    def test_import_dsig_namespace__issue_357(self):
+        location = 'https://www.w3.org/TR/2008/REC-xmldsig-core-20080610/xmldsig-core-schema.xsd'
+        dsig_namespace = 'http://www.w3.org/2000/09/xmldsig#'
+
+        schema = self.schema_class(dedent(f"""<?xml version="1.0" encoding="UTF-8"?>
+            <!-- Test import of defused data from remote with a fallback.-->
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
+                <xs:import namespace="{dsig_namespace}"
+                    schemaLocation="{location}"/>
+                <xs:element name="root"/>
+            </xs:schema>"""))
+
+        self.assertIn(dsig_namespace, schema.maps.namespaces)
+        url = schema.maps.namespaces[dsig_namespace][0].url
+        self.assertIsInstance(url, str)
+        self.assertTrue(url.endswith('schemas/DSIG/xmldsig-core-schema.xsd'))
+
+    def test_include_overlap(self):
+        schema = self.schema_class(dedent("""\
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
+                <xs:element name="elem1"/>
+                <xs:element name="elem2"/>
+            </xs:schema>"""))
+
+        with self.assertRaises(XMLSchemaParseError) as ctx:
+            self.schema_class(dedent("""\
+                <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
+                    <xs:element name="elem1"/>
+                    <xs:element name="elem2"/>
+                </xs:schema>"""), global_maps=schema.maps)
+
+        self.assertIn("global element with name='elem1' is already defined",
+                      str(ctx.exception))
+
+    def test_use_xpath3(self):
+        schema = self.schema_class(dedent("""\
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
+                <xs:element name="root"/>
+            </xs:schema>"""), use_xpath3=True)
+
+        self.assertFalse(schema.use_xpath3)
+
+    def test_xmlns_namespace_forbidden(self):
+        source = dedent("""\
+             <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema"
+                     targetNamespace="http://www.w3.org/2000/xmlns/">
+                 <xs:element name="root"/>
+             </xs:schema>""")
+
+        with self.assertRaises(ValueError) as ctx:
+            self.schema_class(source)
+
+        self.assertIn('http://www.w3.org/2000/xmlns/', str(ctx.exception))
+
 
 class TestXMLSchema11(TestXMLSchema10):
 
@@ -889,6 +813,37 @@ class TestXMLSchema11(TestXMLSchema10):
                 </xs:schema>"""))
         self.assertEqual("prefix 'x' not found in namespace map", ctx.exception.message)
 
+    def test_use_xpath3(self):
+        schema = self.schema_class(dedent("""\
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
+                <xs:element name="root"/>
+            </xs:schema>"""), use_xpath3=True)
+
+        self.assertTrue(schema.use_xpath3)
+
+        schema = self.schema_class(dedent("""\
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
+                <xs:element name="root" type="rootType"/>
+                <xs:complexType name="rootType">
+                                  <xs:assert test="let $foo := 'bar' return $foo"/>
+
+                </xs:complexType>
+            </xs:schema>"""), use_xpath3=True)
+
+        self.assertTrue(schema.use_xpath3)
+
+        with self.assertRaises(XMLSchemaParseError) as ctx:
+            self.schema_class(dedent("""\
+                <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
+                    <xs:element name="root" type="rootType"/>
+                    <xs:complexType name="rootType">
+                                      <xs:assert test="let $foo := 'bar' return $foo"/>
+
+                    </xs:complexType>
+                </xs:schema>"""))
+
+        self.assertIn('XPST0003', str(ctx.exception))
+
 
 class TestXMLSchemaMeta(unittest.TestCase):
 
